package net.chenlin.dp.modules.basics.controller;

import net.chenlin.dp.common.annotation.RestAnon;
import net.chenlin.dp.common.annotation.SysLog;
import net.chenlin.dp.common.entity.Page;
import net.chenlin.dp.common.entity.R;
import net.chenlin.dp.common.excel.BusClick;
import net.chenlin.dp.common.excel.ExcelUtils;
import net.chenlin.dp.modules.basics.entity.BasicsPersonnelEntity;
import net.chenlin.dp.modules.basics.entity.BasicsPersonnelFillinEntity;
import net.chenlin.dp.modules.basics.service.BasicsPersonnelFillinService;
import net.chenlin.dp.modules.sys.controller.AbstractController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 微信小程序调用接口
 * @author ZhouChenglin<yczclcn@163.com>
 */
@RestController
@RequestMapping("/rest/interface")
public class InterfaceController extends AbstractController {
	
	@Autowired
	private BasicsPersonnelFillinService basicsPersonnelFillinService;
	
	/**
	 * 列表
	 * @param params
	 * @return
	 */
	@RequestMapping("/list")
	public Page<BasicsPersonnelFillinEntity> list(@RequestBody Map<String, Object> params) {
		return basicsPersonnelFillinService.listBasicsPersonnelFillin(params);
	}
		
	/**
	 * 新增
	 * @param basicsPersonnelFillin
	 * @return
	 */
	@RestAnon
	@SysLog("新增人员每日填报温度信息")
	@RequestMapping(value="save",method = RequestMethod.POST)
	public R save(@RequestBody BasicsPersonnelFillinEntity basicsPersonnelFillin) {
		basicsPersonnelFillin.setGmtCreate(new Date());
		return basicsPersonnelFillinService.saveBasicsPersonnelFillin(basicsPersonnelFillin);
	}
	
	/**
	 * 根据id查询详情
	 * @param id
	 * @return
	 */
	@RequestMapping("/info")
	public R getById(@RequestBody Long id) {
		return basicsPersonnelFillinService.getBasicsPersonnelFillinById(id);
	}

	/**
	 * 根据id查询详情
	 * @param id
	 * @return
	 */
	@RestAnon
	@RequestMapping("/getByUserCode")
	public List<BasicsPersonnelEntity> getByUserCode(String id) {
		return basicsPersonnelFillinService.getByUserCode(id);
	}

	/**
	 * 修改
	 * @param basicsPersonnelFillin
	 * @return
	 */
	@SysLog("修改人员每日填报温度信息")
	@RequestMapping("/update")
	public R update(@RequestBody BasicsPersonnelFillinEntity basicsPersonnelFillin) {
		return basicsPersonnelFillinService.updateBasicsPersonnelFillin(basicsPersonnelFillin);
	}
	
	/**
	 * 删除
	 * @param id
	 * @return
	 */
	@SysLog("删除人员每日填报温度信息")
	@RequestMapping("/remove")
	public R batchRemove(@RequestBody Long[] id) {
		return basicsPersonnelFillinService.batchRemove(id);
	}

	@RestAnon
	@RequestMapping(value = "/exportExcel", method = RequestMethod.GET)
	public void exportExcel(HttpServletResponse response)  throws IOException {
		List<BusClick> resultList = new ArrayList<BusClick>();
		BusClick busClick = new BusClick();
		busClick.setCityCode("a1");
		busClick.setClientVer("a2");
		busClick.setDate("a3");
		busClick.setMarkId("a4");
		busClick.setToaluv("a5");
		resultList.add(busClick);

		busClick = new BusClick();
		busClick.setCityCode("b1");
		busClick.setClientVer("b2");
		busClick.setDate("b3");
		busClick.setMarkId("b4");
		busClick.setToaluv("b5");
		resultList.add(busClick);

		long t1 = System.currentTimeMillis();
		ExcelUtils.writeExcel("导出.xlsx",response, resultList, BusClick.class);
		long t2 = System.currentTimeMillis();
		System.out.println(String.format("write over! cost:%sms", (t2 - t1)));
	}
	
}
